# Python Version: 3.x
__author__  = 'Kimiyuki Onaka'
__email__ = 'kimiyuki95@gmail.com'
__license__ = 'MIT License'
__url__ = 'https://github.com/kmyk/online-judge-tools'
__version_info__ = ( 0, 1, 36, 'final', 0 )
__version__ = '.'.join(map(str, __version_info__[: 3]))
